#!/usr/bin/env python
# __main__.py

import codecs
import configparser
import importlib
import os
import re
import sys
import time
from functools import update_wrapper

import click
import requests

from apigee import (APIGEE_CLI_EXCEPTIONS_LOG_FILE,
                    APIGEE_CLI_PLUGINS_DIRECTORY, APIGEE_CLI_PLUGINS_PATH, APP,
                    CMD)
from apigee import __version__ as version
from apigee.apiproducts.commands import apiproducts
from apigee.apis.commands import apis
from apigee.apps.commands import apps
from apigee.auth import auth
from apigee.backups.commands import backups
from apigee.caches.commands import caches
from apigee.cls import AliasedGroup
from apigee.configure.commands import configure
from apigee.deployments.commands import deployments
from apigee.developers.commands import developers
from apigee.exceptions import exception_handler, setup_global_logger
from apigee.keystores.commands import keystores
from apigee.keyvaluemaps.commands import keyvaluemaps
from apigee.maskconfigs.commands import maskconfigs
from apigee.permissions.commands import permissions
from apigee.plugins.commands import plugins
from apigee.references.commands import references
from apigee.sharedflows.commands import sharedflows
from apigee.targetservers.commands import targetservers
from apigee.userroles.commands import userroles
from apigee.utils import (import_all_modules_in_directory, is_dir,
                          run_func_on_dir_files, show_message)
from apigee.virtualhosts.commands import virtualhosts

# from click_aliases import ClickAliasedGroup


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(
    context_settings=CONTEXT_SETTINGS, cls=AliasedGroup, invoke_without_command=False, chain=False
)
# @click.group(context_settings=CONTEXT_SETTINGS, cls=ClickAliasedGroup, invoke_without_command=False, chain=False)
@click.version_option(version, '-V', '--version')
@click.pass_context
def cli(ctx):
    """Welcome to the (Unofficial) Apigee Management API command-line interface!

    \b
    Docs:    https://darumatic.github.io/apigee-cli/
    PyPI:    https://pypi.org/project/apigeecli/
    GitHub:  https://github.com/darumatic/apigee-cli

    \f

    :param click.core.Context ctx: Click context.
    """
    # ensure that ctx.obj exists and is a dict (in case `cli()` is called
    # by means other than the `if` block below)
    ctx.ensure_object(dict)


@exception_handler
def main():
    setup_global_logger(APIGEE_CLI_EXCEPTIONS_LOG_FILE)

    cli_commands = {
        backups,
        configure,
        deployments,
        caches,
        keyvaluemaps,
        targetservers,
        apis,
        apiproducts,
        apps,
        developers,
        auth,
        maskconfigs,
        userroles,
        permissions,
        sharedflows,
        keystores,
        references,
        virtualhosts,
        plugins,
    }

    run_func_on_dir_files(
        APIGEE_CLI_PLUGINS_DIRECTORY,
        import_all_modules_in_directory,
        args=(cli_commands,),
        glob='[!.][!__]*/__init__.py',
    )

    for command in cli_commands:
        cli.add_command(command)

    cli(prog_name=CMD, obj={})


if __name__ == '__main__':
    # cli() # pragma: no cover
    main()  # pragma: no cover



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
# from apigee.cls import OptionEatAll
# from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.userroles.userroles import Userroles
from apigee.verbose import common_verbose_options


@click.group(
    help='Roles for users in an organization on Apigee Edge. User roles form the basis of role-based access in Apigee Edge. Users are associated with one or more userroles. Each userrole defines a set of permissions (GET, PUT, DELETE) on RBAC resources (defined by URI paths).'
)
def userroles():
    pass


def _add_a_user_to_a_role(
    username, password, mfa_secret, token, zonename, org, profile, name, user_email, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .add_a_user_to_a_role(user_email)
        .text
    )


@userroles.command(help='Add a user to a role.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--user-email', help='the email address of the user', required=True)
def add_user(*args, **kwargs):
    console.echo(_add_a_user_to_a_role(*args, **kwargs))


def _add_permissions_for_a_resource_to_a_user_role(
    username, password, mfa_secret, token, zonename, org, profile, name, body, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .add_permissions_for_a_resource_to_a_user_role(body)
        .text
    )


@userroles.command(help='Add Permissions for Resource to a Role')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('-b', '--body', help='request body', required=True)
def add_permissions(*args, **kwargs):
    console.echo(_add_permissions_for_a_resource_to_a_user_role(*args, **kwargs))


def _add_permissions_for_multiple_resources_to_a_user_role(
    username, password, mfa_secret, token, zonename, org, profile, name, body, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .add_permissions_for_multiple_resources_to_a_user_role(body)
        .text
    )


@userroles.command(help='Adds multiple permissions to multiple resources simultaneously.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('-b', '--body', help='request body', required=True)
def add_permissions_multiple(*args, **kwargs):
    console.echo(_add_permissions_for_multiple_resources_to_a_user_role(*args, **kwargs))


def _create_a_user_role_in_an_organization(
    username, password, mfa_secret, token, zonename, org, profile, names, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, None)
        .create_a_user_role_in_an_organization(names)
        .text
    )


@userroles.command(help='Creates one or more user roles in an organization.')
@common_auth_options
@common_verbose_options
@common_silent_options
# @click.option(
#     '-n', '--names', metavar='LIST', cls=OptionEatAll, help='list of role names', required=True
# )
@click.option('-n', '--names', multiple=True, help='role names', required=True)
def create(*args, **kwargs):
    console.echo(_create_a_user_role_in_an_organization(*args, **kwargs))


def _delete_a_permission_for_a_resource(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    permission,
    resource_path,
    **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_a_permission_for_a_resource(permission, resource_path)
        .text
    )


@userroles.command(
    help='Removes a permission from a resource for the role specified. Permissions are case sensitive. Specify the permission as get, put, or delete.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--permission', help='get, put, or delete', required=True)
@click.option('--resource-path', help='the resource path', required=True)
def delete_permission(*args, **kwargs):
    console.echo(_delete_a_permission_for_a_resource(*args, **kwargs))


def _delete_resource_from_permissions(
    username, password, mfa_secret, token, zonename, org, profile, name, resource_path, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_resource_from_permissions(resource_path)
        .text
    )


@userroles.command(
    help='Removes all permissions for a resource for the role specified. Permissions are case sensitive.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--resource-path', help='the resource path', required=True)
def delete_resource(*args, **kwargs):
    console.echo(_delete_resource_from_permissions(*args, **kwargs))


def _delete_a_user_role(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_a_user_role()
        .text
    )


@userroles.command(
    help='Deletes a role from an organization. Roles can only be deleted when no users are in the role.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
def delete(*args, **kwargs):
    console.echo(_delete_a_user_role(*args, **kwargs))


def _get_a_role(username, password, mfa_secret, token, zonename, org, profile, name, **kwargs):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_a_role()
        .text
    )


@userroles.command(help='Gets the name of a user role.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
def get(*args, **kwargs):
    console.echo(_get_a_role(*args, **kwargs))


def _get_resource_permissions_for_a_specific_role(
    username, password, mfa_secret, token, zonename, org, profile, name, resource_path="", **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_resource_permissions_for_a_specific_role(resource_path=resource_path)
        .text
    )


@userroles.command(help='Gets a list of permissions associated with the specified resource.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--resource-path', help='the resource path', required=True)
def get_permissions(*args, **kwargs):
    console.echo(_get_resource_permissions_for_a_specific_role(*args, **kwargs))


def _get_users_for_a_role(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_users_for_a_role()
        .text
    )


@userroles.command(help='Returns a list of all system users associated with a role.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
def get_users(*args, **kwargs):
    console.echo(_get_users_for_a_role(*args, **kwargs))


def _list_permissions_for_a_resource(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .list_permissions_for_a_resource()
        .text
    )


@userroles.command(help='Gets permissions for all resources associated with a user role.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
def list_permissions(*args, **kwargs):
    console.echo(_list_permissions_for_a_resource(*args, **kwargs))


def _list_user_roles(username, password, mfa_secret, token, zonename, org, profile, **kwargs):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, None)
        .list_user_roles()
        .text
    )


@userroles.command(help='Gets a list of roles available to users in an organization.')
@common_auth_options
@common_verbose_options
@common_silent_options
# @common_prefix_options
def list(*args, **kwargs):
    console.echo(_list_user_roles(*args, **kwargs))


def _remove_user_membership_in_role(
    username, password, mfa_secret, token, zonename, org, profile, name, user_email, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .remove_user_membership_in_role(user_email)
        .text
    )


@userroles.command(help='Remove user membership in role.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--user-email', help='the email address of the user', required=True)
def remove_user(*args, **kwargs):
    console.echo(_remove_user_membership_in_role(*args, **kwargs))


def _verify_a_user_roles_permission_on_a_specific_RBAC_resource(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    permission,
    resource_path,
    **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .verify_a_user_roles_permission_on_a_specific_RBAC_resource(permission, resource_path)
        .text
    )


@userroles.command(
    help="Verifies that a user role's permission on a specific resource exists. Returns a value of true or false."
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--permission', help='get, put, or delete', required=True)
@click.option('--resource-path', help='the resource path', required=True)
def verify_permission(*args, **kwargs):
    console.echo(_verify_a_user_roles_permission_on_a_specific_RBAC_resource(*args, **kwargs))


def _verify_user_role_membership(
    username, password, mfa_secret, token, zonename, org, profile, name, user_email, **kwargs
):
    return (
        Userroles(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .verify_user_role_membership(user_email)
        .text
    )


@userroles.command(help='Verify user role membership.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option('--user-email', help='the email address of the user', required=True)
def verify_membership(*args, **kwargs):
    console.echo(_verify_user_role_membership(*args, **kwargs))



import sys

import click

from apigee import APIGEE_CLI_SYMMETRIC_KEY, console
from apigee.auth import common_auth_options, gen_auth
from apigee.crypto import (ENCRYPTED_HEADER_BEGIN, ENCRYPTED_HEADER_END,
                           decrypt_message, encrypt_message, is_encrypted)
from apigee.keyvaluemaps.keyvaluemaps import Keyvaluemaps
from apigee.keyvaluemaps.serializer import KeyvaluemapsSerializer
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.utils import read_file, write_file
from apigee.verbose import common_verbose_options


@click.group(
    help='Key/value maps at the environment scope can be accessed by any API proxy in the environment (such as test or prod). In the management UI (APIs > Environment Configuration), key/value maps are at the environment scope.'
)
def keyvaluemaps():
    pass


def _create_keyvaluemap_in_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_keyvaluemap_in_an_environment(environment, body)
        .text
    )


@keyvaluemaps.command(help='Creates a key value map in an environment.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create(*args, **kwargs):
    console.echo(_create_keyvaluemap_in_an_environment(*args, **kwargs))


def _delete_keyvaluemap_from_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_keyvaluemap_from_an_environment(environment)
        .text
    )


@keyvaluemaps.command(
    help='Deletes a key/value map and all associated entries from an environment.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def delete(*args, **kwargs):
    console.echo(_delete_keyvaluemap_from_an_environment(*args, **kwargs))


def _delete_keyvaluemap_entry_in_an_environment(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    entry_name,
    **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_keyvaluemap_entry_in_an_environment(environment, entry_name)
        .text
    )


@keyvaluemaps.command(
    help='Deletes a specific key/value map entry in an environment by name, along with associated entries.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--entry-name', help='entry name', required=True)
def delete_entry(*args, **kwargs):
    console.echo(_delete_keyvaluemap_entry_in_an_environment(*args, **kwargs))


def _get_keyvaluemap_in_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_keyvaluemap_in_an_environment(environment)
        .text
    )


@keyvaluemaps.command(
    help='Gets a KeyValueMap (KVM) in an environment by name, along with the keys and values.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get(*args, **kwargs):
    console.echo(_get_keyvaluemap_in_an_environment(*args, **kwargs))


def _get_a_keys_value_in_an_environment_scoped_keyvaluemap(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    entry_name,
    **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_a_keys_value_in_an_environment_scoped_keyvaluemap(environment, entry_name)
        .text
    )


@keyvaluemaps.command(help='Gets the value of a key in an environment-scoped KeyValueMap (KVM).')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--entry-name', help='entry name', required=True)
def get_value(*args, **kwargs):
    console.echo(_get_a_keys_value_in_an_environment_scoped_keyvaluemap(*args, **kwargs))


def _list_keyvaluemaps_in_an_environment(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    prefix=None,
    **kwargs
):
    return Keyvaluemaps(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).list_keyvaluemaps_in_an_environment(environment, prefix=prefix)


@keyvaluemaps.command(
    help='Lists the name of all key/value maps in an environment and optionally returns an expanded view of all key/value maps for the environment.'
)
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
def list(*args, **kwargs):
    console.echo(_list_keyvaluemaps_in_an_environment(*args, **kwargs))


def _update_keyvaluemap_in_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .update_keyvaluemap_in_an_environment(environment, body)
        .text
    )


@keyvaluemaps.command(
    help='Note: This API is supported for Apigee Edge for Private Cloud only. For Apigee Edge for Public Cloud use Update an entry in an environment-scoped KVM. Updates an existing KeyValueMap in an environment. Does not override the existing map. Instead, this method updates the entries if they exist or adds them if not. It can take several minutes before the new value is visible to runtime traffic.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def update(*args, **kwargs):
    console.echo(_update_keyvaluemap_in_an_environment(*args, **kwargs))


def _create_an_entry_in_an_environment_scoped_kvm(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    entry_name,
    entry_value,
    **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_an_entry_in_an_environment_scoped_kvm(environment, entry_name, entry_value)
        .text
    )


@keyvaluemaps.command(
    help='Note: This API is supported for Apigee Edge for the Public Cloud only. Creates an entry in an existing KeyValueMap scoped to an environment. A key (name) cannot be larger than 2 KB. KVM names are case sensitive.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--entry-name', help='entry name', required=True)
@click.option('--entry-value', help='entry value', required=True)
def create_entry(*args, **kwargs):
    console.echo(_create_an_entry_in_an_environment_scoped_kvm(*args, **kwargs))


def _update_an_entry_in_an_environment_scoped_kvm(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    entry_name,
    updated_value,
    **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .update_an_entry_in_an_environment_scoped_kvm(environment, entry_name, updated_value)
        .text
    )


@keyvaluemaps.command(
    help='Note: This API is supported for Apigee Edge for the Public Cloud only. Updates an entry in a KeyValueMap scoped to an environment. A key cannot be larger than 2 KB. KVM names are case sensitive. Does not override the existing map. It can take several minutes before the new value is visible to runtime traffic.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--entry-name', help='entry name', required=True)
@click.option('--updated-value', help='updated value', required=True)
def update_entry(*args, **kwargs):
    console.echo(_update_an_entry_in_an_environment_scoped_kvm(*args, **kwargs))


def _list_keys_in_an_environment_scoped_keyvaluemap(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    startkey,
    count,
    **kwargs
):
    return (
        Keyvaluemaps(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .list_keys_in_an_environment_scoped_keyvaluemap(environment, startkey, count)
        .text
    )


@keyvaluemaps.command(
    help='Note: This API is supported for Apigee Edge for the Public Cloud only. Lists keys in a KeyValueMap scoped to an environment. KVM names are case sensitive.'
)
@common_auth_options
# @common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option(
    '--startkey',
    default="",
    show_default=True,
    help='To filter the keys that are returned, enter the name of a key that the list will start with.',
)
@click.option(
    '--count',
    type=click.INT,
    default=100,
    show_default=True,
    help='Limits the list of keys to the number you specify, up to a maximum of 100. Use with the startkey parameter to provide more targeted filtering.',
)
# @click.option("--prefix", help="team/resource prefix filter")
def list_keys(*args, **kwargs):
    console.echo(_list_keys_in_an_environment_scoped_keyvaluemap(*args, **kwargs))


def _push_keyvaluemap(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    file,
    symmetric_key,
    **kwargs
):
    return Keyvaluemaps(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).push_keyvaluemap(environment, file, secret=symmetric_key)


@keyvaluemaps.command(
    help='Push KeyValueMap to Apigee. This will create KeyValueMap/entries if they do not exist, update existing KeyValueMap/entries, and delete entries on Apigee that are not present in the request body.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
@click.option(
    '--symmetric-key', default=APIGEE_CLI_SYMMETRIC_KEY, help='symmetric secret key for decrypting'
)
def push(*args, **kwargs):
    _push_keyvaluemap(*args, **kwargs)


@keyvaluemaps.command(
    name='encrypt', help='Use symmetric GPG (AES256) to encrypt KVM file in a custom format.'
)
@common_silent_options
@common_verbose_options
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
@click.option('--symmetric-key', required=True, help='symmetric secret key for encrypting')
def encrypt_file(symmetric_key, file, verbose, silent):
    contents = read_file(file, type='json')
    encrypted_count = 0
    console.echo('Encrypting... ', end='', flush=True)
    contents, encrypted_count = KeyvaluemapsSerializer().encrypt_decrypt_keyvaluemap(
        contents, symmetric_key, encrypt=True
    )
    if encrypted_count:
        write_file(contents, file, indent=2)
        console.echo('Done')
        return contents
    console.echo('None')
    return ''


@keyvaluemaps.command(
    name='decrypt', help='Use symmetric GPG (AES256) to decrypt KVM file in a custom format.'
)
@common_silent_options
@common_verbose_options
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
@click.option('--symmetric-key', required=True, help='symmetric secret key for decrypting')
def decrypt_file(symmetric_key, file, verbose, silent):
    contents = read_file(file, type='json')
    decrypted_count = 0
    console.echo('Decrypting... ', end='', flush=True)
    contents, decrypted_count = KeyvaluemapsSerializer().encrypt_decrypt_keyvaluemap(
        contents, symmetric_key, encrypt=False
    )
    if decrypted_count:
        write_file(contents, file, indent=2)
        console.echo('Done')
        return contents
    console.echo('None')
    return ''



import json

import requests
from tabulate import tabulate

from apigee import APIGEE_ADMIN_API_URL, auth
from apigee.deployments.serializer import DeploymentsSerializer

GET_API_PROXY_DEPLOYMENT_DETAILS_PATH = (
    '{api_url}/v1/organizations/{org}/apis/{api_name}/deployments'
)


class Deployments:
    def __init__(self, auth, org_name, api_name):
        self._auth = auth
        self._org_name = org_name
        self._api_name = api_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def api_name(self):
        return self._api_name

    @api_name.setter
    def api_name(self, value):
        self._api_name = value

    def get_api_proxy_deployment_details(
        self,
        formatted=False,
        format='text',
        showindex=False,
        tablefmt='plain',
        revision_name_only=False,
    ):
        uri = GET_API_PROXY_DEPLOYMENT_DETAILS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, api_name=self._api_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        if formatted:
            if revision_name_only:
                return DeploymentsSerializer().serialize_details(
                    resp, format, showindex=showindex, tablefmt=tablefmt
                )
            return DeploymentsSerializer().serialize_details(resp, 'text')
        return resp



import json


class VirtualhostsSerializer:
    def serialize_details(self, virtualhosts, format, prefix=None):
        resp = virtualhosts
        if format == 'text':
            return virtualhosts.text
        virtualhosts = virtualhosts.json()
        if prefix:
            virtualhosts = [
                virtualhost for virtualhost in virtualhosts if virtualhost.startswith(prefix)
            ]
        if format == 'json':
            return json.dumps(virtualhosts)
        elif format == 'table':
            pass
        elif format == 'dict':
            return virtualhosts
        return resp



import json


class ReferencesSerializer:
    def serialize_details(self, references, format, prefix=None):
        resp = references
        if format == 'text':
            return references.text
        references = references.json()
        if prefix:
            references = [reference for reference in references if reference.startswith(prefix)]
        if format == 'json':
            return json.dumps(references)
        elif format == 'table':
            pass
        elif format == 'dict':
            return references
        return resp



import json

import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.keystores.serializer import KeystoresSerializer

CREATE_A_KEYSTORE_OR_TRUSTSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores'
)
DELETE_A_KEYSTORE_OR_TRUSTSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}'
)
LIST_ALL_KEYSTORES_AND_TRUSTSTORES_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores'
)
GET_A_KEYSTORE_OR_TRUSTSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}'
)
TEST_A_KEYSTORE_OR_TRUSTSTORE_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/testssl'
GET_CERT_DETAILS_FROM_A_KEYSTORE_OR_TRUSTSTORE_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/certs/{cert_name}'
GET_ALL_CERTS_FROM_A_KEYSTORE_OR_TRUSTSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/certs'
)
DELETE_CERT_FROM_A_KEYSTORE_OR_TRUSTSTORE_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/certs/{cert_name}'
EXPORT_A_CERT_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/certs/{cert_name}/export'
UPLOAD_A_CERTIFICATE_TO_A_TRUSTSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/certs'
)
UPLOAD_A_JAR_FILE_TO_A_KEYSTORE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/keys'
)
CREATE_AN_ALIAS_BY_GENERATING_A_SELF_SIGNED_CERTIFICATE_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases'
)
LIST_ALIASES_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases'
)
GET_ALIAS_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases/{alias_name}'
UPDATE_THE_CERTIFICATE_IN_AN_ALIAS_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases/{alias_name}'
GENERATE_A_CSR_FOR_AN_ALIAS_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases/{alias_name}/csr'
EXPORT_A_CERTIFICATE_FOR_AN_ALIAS_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases/{alias_name}/certificate'
DELETE_ALIAS_PATH = '{api_url}/v1/o/{org_name}/environments/{environment}/keystores/{keystore_name}/aliases/{alias_name}'


class Keystores:
    def __init__(self, auth, org_name, keystore_name):
        self._auth = auth
        self._org_name = org_name
        self._keystore_name = keystore_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def keystore_name(self):
        return self._keystore_name

    @keystore_name.setter
    def keystore_name(self, value):
        self._keystore_name = value

    def create_a_keystore_or_truststore(self):
        pass

    def delete_a_keystore_or_truststore(self):
        pass

    def list_all_keystores_and_truststores(self, environment, prefix=None, format='json'):
        uri = LIST_ALL_KEYSTORES_AND_TRUSTSTORES_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org_name=self._org_name, environment=environment
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return KeystoresSerializer().serialize_details(resp, format, prefix=prefix)

    def get_a_keystore_or_truststore(self, environment):
        uri = GET_A_KEYSTORE_OR_TRUSTSTORE_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def test_a_keystore_or_truststore(self):
        pass

    def get_cert_details_from_a_keystore_or_truststore(self, environment, cert_name):
        uri = GET_CERT_DETAILS_FROM_A_KEYSTORE_OR_TRUSTSTORE_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
            cert_name=cert_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_all_certs_from_a_keystore_or_truststore(self, environment, prefix=None, format='json'):
        uri = GET_ALL_CERTS_FROM_A_KEYSTORE_OR_TRUSTSTORE_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return KeystoresSerializer().serialize_details(resp, format, prefix=prefix)

    def delete_cert_from_a_keystore_or_truststore(self):
        pass

    def export_a_cert(self, environment, cert_name):
        uri = EXPORT_A_CERT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
            cert_name=cert_name,
        )
        hdrs = auth.set_header(self._auth, headers={})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def upload_a_certificate_to_a_truststore(self):
        pass

    def upload_a_jar_file_to_a_keystore(self):
        pass

    def create_an_alias_by_generating_a_self_signed_certificate(self):
        pass

    def list_aliases(self, environment, prefix=None, format='json'):
        uri = LIST_ALIASES_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return KeystoresSerializer().serialize_details(resp, format, prefix=prefix)

    def get_alias(self, environment, alias_name):
        uri = GET_ALIAS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
            alias_name=alias_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def update_the_certificate_in_an_alias(self):
        pass

    def generate_a_csr_for_an_alias(self):
        pass

    def export_a_certificate_for_an_alias(self, environment, alias_name):
        uri = EXPORT_A_CERTIFICATE_FOR_AN_ALIAS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            keystore_name=self._keystore_name,
            alias_name=alias_name,
        )
        hdrs = auth.set_header(self._auth, headers={})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_alias(self):
        pass



import click
from click_aliases import ClickAliasedGroup

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.permissions.permissions import Permissions
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options

TABLEFMT_CHOICES = [
    'plain',
    'simple',
    'github',
    'grid',
    'fancy_grid',
    'pipe',
    'orgtbl',
    'jira',
    'presto',
    'psql',
    'rst',
    'mediawiki',
    'moinmoin',
    'youtrack',
    'html',
    'latex',
    'latex_raw',
    'latex_booktabs',
    'textile',
]


@click.group(help='Permissions for roles in an organization on Apigee Edge.', cls=ClickAliasedGroup)
def permissions():
    pass


def _create_permissions(
    username, password, mfa_secret, token, zonename, org, profile, name, body, **kwargs
):
    return (
        Permissions(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_permissions(body)
        .text
    )


@permissions.command(help='Create permissions for a role.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create(*args, **kwargs):
    console.echo(_create_permissions(*args, **kwargs))


def _team_permissions(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    file,
    placeholder_key=None,
    placeholder_value="",
    **kwargs
):
    return (
        Permissions(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .team_permissions(
            file, placeholder_key=placeholder_key, placeholder_value=placeholder_value
        )
        .text
    )


@permissions.command(
    help='Create permissions for a role using a template file.', aliases=['template-permissions']
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='the role name', required=True)
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
@click.option(
    '--placeholder-key', default=None, help='placeholder key to replace with a placeholder value'
)
@click.option(
    '--placeholder-value',
    default="",
    show_default=True,
    help='placeholder value to replace placeholder key.',
)
def template(*args, **kwargs):
    console.echo(_team_permissions(*args, **kwargs))


def _get_permissions(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    format,
    showindex=False,
    tablefmt='plain',
    **kwargs
):
    return Permissions(
        gen_auth(username, password, mfa_secret, token, zonename), org, name
    ).get_permissions(
        formatted=True,
        format='text' if format == 'json' else format,
        showindex=showindex,
        tablefmt=tablefmt,
    )


@permissions.command(help='Get permissions for a role.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
# @click.option("-j", "--json", help="display json output when using -r flag", default="table")
@click.option(
    '--format',
    help='defines how to format output',
    default='table',
    type=click.Choice(['json', 'table'], case_sensitive=False),
)
@click.option('--showindex/--no-showindex', default=False)
@click.option(
    '--tablefmt',
    help='defines how the table is formatted',
    type=click.Choice(TABLEFMT_CHOICES, case_sensitive=False),
    default='plain',
    show_default=True,
)
def get(*args, **kwargs):
    console.echo(_get_permissions(*args, **kwargs))



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.caches.caches import Caches
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options


@click.group(
    help='A lightweight persistence store that can be used by policies or code executing on the Apigee Edge. To support data segregation, cache resources are scoped to environments.'
)
def caches():
    pass


def _clear_all_cache_entries(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .clear_all_cache_entries(environment)
        .text
    )


@caches.command(help='Clears all cache entries.')
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@common_auth_options
@common_silent_options
@common_verbose_options
def clear(*args, **kwargs):
    console.echo(_clear_all_cache_entries(*args, **kwargs))


def _clear_a_cache_entry(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    entry,
    **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .clear_a_cache_entry(environment, entry)
        .text
    )


@caches.command(
    help='Clears a cache entry, which is identified by the full CacheKey prefix and value.'
)
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--entry', help='cache entry to clear', required=True)
@common_auth_options
@common_silent_options
@common_verbose_options
def clear_entry(*args, **kwargs):
    console.echo(_clear_a_cache_entry(*args, **kwargs))


def _create_a_cache_in_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_a_cache_in_an_environment(environment, body)
        .text
    )


@caches.command(
    help="Creates a cache in an environment. Caches are created per environment. For data segregation, a cache created in 'test', for example, cannot be accessed by API proxies deployed in 'prod'. The JSON object in the request body can be empty to create a cache with the default settings."
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create(*args, **kwargs):
    console.echo(_create_a_cache_in_an_environment(*args, **kwargs))


def _get_information_about_a_cache(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_information_about_a_cache(environment)
        .text
    )


@caches.command(
    help='Gets information about a cache. The response might contain a property named persistent. That property is no longer used by Edge.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get(*args, **kwargs):
    console.echo(_get_information_about_a_cache(*args, **kwargs))


def _list_caches_in_an_environment(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    prefix=None,
    **kwargs
):
    return Caches(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).list_caches_in_an_environment(environment, prefix=prefix)


@caches.command(help='List caches in an environment.')
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
def list(*args, **kwargs):
    console.echo(_list_caches_in_an_environment(*args, **kwargs))


def _update_a_cache_in_an_environment(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .update_a_cache_in_an_environment(environment, body)
        .text
    )


@caches.command(
    help='Updates a cache in an environment. You must specify the complete definition of the cache, including the properties that you want to change and the ones that retain their current value. Any properties omitted from the request body are reset to their default value. Use Get information about a cache to obtain an object containing the current value of all properties, then change only those that you want to update.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def update(*args, **kwargs):
    console.echo(_update_a_cache_in_an_environment(*args, **kwargs))


def _delete_a_cache(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Caches(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_a_cache(environment)
        .text
    )


@caches.command(help='Deletes a cache.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def delete(*args, **kwargs):
    console.echo(_delete_a_cache(*args, **kwargs))


def _push_cache(
    username, password, mfa_secret, token, zonename, org, profile, environment, file, **kwargs
):
    return Caches(gen_auth(username, password, mfa_secret, token, zonename), org, None).push_cache(
        environment, file
    )


@caches.command(help='Push Cache to Apigee. This will create/update a Cache.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
def push(*args, **kwargs):
    _push_cache(*args, **kwargs)



import json

from apigee.utils import remove_last_items_from_list, run_func_on_iterable


class ApisSerializer:
    def filter_deployed_revisions(self, details):
        return list(set(run_func_on_iterable(details, lambda d: d['revision'], state_op='extend')))

    def filter_deployment_details(self, details):
        return run_func_on_iterable(
            details['environment'],
            lambda d: {
                'name': d['name'],
                'revision': [revision['name'] for revision in d['revision']],
            },
        )

    def filter_undeployed_revisions(self, revisions, deployed, save_last=0):
        return remove_last_items_from_list(
            sorted([int(rev) for rev in revisions if rev not in deployed]), save_last
        )

    def serialize_details(self, apis, format, prefix=None):
        resp = apis
        if format == 'text':
            return apis.text
        apis = apis.json()
        if prefix:
            apis = [api for api in apis if api.startswith(prefix)]
        if format == 'json':
            return json.dumps(apis)
        elif format == 'table':
            pass
        elif format == 'dict':
            return apis
        return resp



import click


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx) if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))


class OptionEatAll(click.Option):
    """https://stackoverflow.com/a/48394004"""

    pass



import base64
import sys

import gnupg

ENCRYPTED_HEADER_BEGIN = '-----BEGIN ENCRYPTED APIGEE CLI MESSAGE-----'
ENCRYPTED_HEADER_END = '-----END ENCRYPTED APIGEE CLI MESSAGE-----'


def encrypt_message(secret, message, encoded=True):
    gpg = gnupg.GPG()
    if encoded:
        return base64.b64encode(
            str(
                gpg.encrypt(message, symmetric='AES256', passphrase=secret, recipients=None)
            ).encode()
        ).decode()
    return str(gpg.encrypt(message, symmetric='AES256', passphrase=secret, recipients=None))


def is_encrypted(message):
    return message.startswith(ENCRYPTED_HEADER_BEGIN) and message.endswith(ENCRYPTED_HEADER_END)


def decrypt_message(secret, message, encoded=True):
    gpg = gnupg.GPG()
    if not is_encrypted(message):
        return ''
    message = message[len(ENCRYPTED_HEADER_BEGIN) : -len(ENCRYPTED_HEADER_END)]
    if encoded:
        return str(gpg.decrypt(base64.b64decode(message).decode(), passphrase=secret))
    return str(gpg.decrypt(message, passphrase=secret))



import json
import random
import string

import requests
from requests.exceptions import HTTPError
from tqdm import tqdm

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.apps.serializer import AppsSerializer

CREATE_DEVELOPER_APP_PATH = '{api_url}/v1/organizations/{org}/developers/{developer}/apps'
DELETE_DEVELOPER_APP_PATH = '{api_url}/v1/organizations/{org}/developers/{developer}/apps/{name}'
CREATE_EMPTY_DEVELOPER_APP_PATH = CREATE_DEVELOPER_APP_PATH
GET_DEVELOPER_APP_DETAILS_PATH = (
    '{api_url}/v1/organizations/{org}/developers/{developer}/apps/{name}'
)
LIST_DEVELOPER_APPS_PATH = '{api_url}/v1/organizations/{org}/developers/{developer}/apps'
DELETE_KEY_FOR_A_DEVELOPER_APP_PATH = (
    '{api_url}/v1/organizations/{org}/developers/{developer}/apps/{name}/keys/{consumer_key}'
)
CREATE_A_CONSUMER_KEY_AND_SECRET_PATH = (
    '{api_url}/v1/organizations/{org}/developers/{developer}/apps/{name}/keys/create'
)
ADD_API_PRODUCT_TO_KEY_PATH = (
    '{api_url}/v1/organizations/{org}/developers/{developer}/apps/{name}/keys/{consumer_key}'
)


class Apps:
    def __init__(self, auth, org_name, app_name):
        self._auth = auth
        self._org_name = org_name
        self._app_name = app_name

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def app_name(self):
        return self._app_name

    @app_name.setter
    def app_name(self, value):
        self._app_name = value

    def __call__(self):
        pass

    def create_developer_app(self, developer, request_body):
        uri = CREATE_DEVELOPER_APP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, developer=developer
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_developer_app(self, developer):
        uri = DELETE_DEVELOPER_APP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            developer=developer,
            name=self._app_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def create_empty_developer_app(self, developer, display_name="", callback_url=""):
        uri = CREATE_EMPTY_DEVELOPER_APP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, developer=developer
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = {
            'name': self._app_name,
            # "apiProducts": args.products,
            'attributes': [{'name': 'DisplayName', 'value': display_name}],
            # "scopes" : args.scopes,
            'callbackUrl': callback_url,
        }
        if not display_name:
            del body['attributes']
        if not callback_url:
            del body['callbackUrl']
        # body = {k: v for k, v in body.items() if v}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        self.delete_key_for_a_developer_app(developer, resp.json()['credentials'][0]['consumerKey'])
        return self.get_developer_app_details(developer)

    def get_developer_app_details(self, developer):
        uri = GET_DEVELOPER_APP_DETAILS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            developer=developer,
            name=self._app_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_developer_apps(
        self, developer, prefix=None, expand=False, count=1000, startkey="", format='json'
    ):
        uri = LIST_DEVELOPER_APPS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, developer=developer
        )
        if expand:
            uri += f'?expand={expand}'
        else:
            uri += f'?count={count}&startKey={startkey}'
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return AppsSerializer().serialize_details(resp, format, prefix=prefix)

    def list_apps_for_all_developers(
        self,
        list_of_developers,
        prefix=None,
        expand=False,
        count=1000,
        startkey="",
        format='dict',
        progress_bar=False,
    ):
        apps = {}
        for developer in list_of_developers:
            apps[developer] = self.list_developer_apps(
                developer,
                prefix=prefix,
                expand=expand,
                count=count,
                startkey=startkey,
                format=format,
            )
        return apps

    def delete_key_for_a_developer_app(self, developer, consumer_key):
        uri = DELETE_KEY_FOR_A_DEVELOPER_APP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            developer=developer,
            name=self._app_name,
            consumer_key=consumer_key,
        )
        hdrs = auth.set_header(self._auth, {'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def create_a_consumer_key_and_secret(
        self,
        developer,
        consumer_key=None,
        consumer_secret=None,
        key_length=32,
        secret_length=32,
        key_suffix=None,
        key_delimiter='-',
        products=[],
    ):
        uri = CREATE_A_CONSUMER_KEY_AND_SECRET_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            developer=developer,
            name=self._app_name,
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        # app = self.get_developer_app_details(developer)
        def _rand(length):
            return "".join(
                random.SystemRandom().choice(string.ascii_letters + string.digits)
                for _ in range(length)
            )

        if not consumer_key:
            consumer_key = _rand(key_length)
        if not consumer_secret:
            consumer_secret = _rand(secret_length)
        if key_suffix:
            consumer_key += key_delimiter
            consumer_key += key_suffix
        body = {'consumerKey': consumer_key, 'consumerSecret': consumer_secret}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        if products:
            console.echo(resp.text)
            consumer_key = resp.json()['consumerKey']
            request_body = json.dumps(
                {'apiProducts': products, 'attributes': resp.json()['attributes']}
            )
            console.echo(f'Adding API Products {products} to consumerKey {consumer_key}')
            return self.add_api_product_to_key(developer, consumer_key, request_body)
        return resp

    def add_api_product_to_key(self, developer, consumer_key, request_body):
        uri = ADD_API_PRODUCT_TO_KEY_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            developer=developer,
            name=self._app_name,
            consumer_key=consumer_key,
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def restore_app(self, file):
        with open(file, 'r') as f:
            app = json.loads(f.read())
        self._app_name = app['name']
        request_body = {}
        request_body['name'] = app['name']
        request_body['attributes'] = app.get('attributes')
        request_body['scopes'] = app.get('scopes')
        request_body['callbackUrl'] = app.get('callbackUrl')
        request_body = {k: v for k, v in request_body.items() if v}
        resp = self.create_developer_app(app['developerId'], json.dumps(request_body))
        consumer_key = resp.json()['credentials'][0]['consumerKey']
        self.delete_key_for_a_developer_app(app['developerId'], consumer_key)
        if app['credentials']:
            for cred in app['credentials']:
                consumer_key = cred['consumerKey']
                consumer_secret = cred['consumerSecret']
                products = [product['apiproduct'] for product in cred['apiProducts']]
                resp = self.create_a_consumer_key_and_secret(
                    app['developerId'],
                    consumer_key=consumer_key,
                    consumer_secret=consumer_secret,
                    products=products,
                )
        return self.get_developer_app_details(app['developerId'])



import click
from click_option_group import MutuallyExclusiveOptionGroup, optgroup

from apigee import console
from apigee.apis.apis import Apis
from apigee.apis.deploy import deploy as deploy_tool
from apigee.auth import common_auth_options, gen_auth
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.types import Struct
from apigee.verbose import common_verbose_options


@click.group(
    help='The proxy APIs let you perform operations on API proxies, such as create, delete, update, and deploy.'
)
def apis():
    pass


def _delete_api_proxy_revision(
    username, password, mfa_secret, token, zonename, org, profile, name, revision_number, **kwargs
):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .delete_api_proxy_revision(name, revision_number)
        .text
    )


@apis.command(
    help='Deletes a revision of an API proxy and all policies, resources, endpoints, and revisions associated with it. The API proxy revision must be undeployed before you can delete it.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
def delete_revision(*args, **kwargs):
    console.echo(_delete_api_proxy_revision(*args, **kwargs))


def _deploy_api_proxy_revision(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    revision_number,
    delay=0,
    override=False,
    **kwargs,
):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .deploy_api_proxy_revision(
            name, environment, revision_number, delay=delay, override=override
        )
        .text
    )


@apis.command(
    help='Deploys a revision of an existing API proxy to an environment in an organization.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
@click.option(
    '--delay',
    type=click.INT,
    default=0,
    help='Enforces a delay, measured in seconds, before the currently deployed API proxy revision is undeployed and replaced by the new revision that is being deployed. Use this setting to minimize the impact of deployment on in-flight transactions. The default value is 0.',
)
@click.option(
    '--override/--no-override',
    default=False,
    help='Flag that specifies whether to use seamless deployment to ensure zero downtime. Set this flag to "true" to instruct Edge to deploy the new revision fully before undeploying the existing revision. Use in conjunction with the delay parameter to control when the existing revision is undeployed.',
)
def deploy_revision(*args, **kwargs):
    console.echo(_deploy_api_proxy_revision(*args, **kwargs))


def _delete_undeployed_revisions(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    save_last=0,
    dry_run=False,
    **kwargs,
):
    return Apis(
        gen_auth(username, password, mfa_secret, token, zonename), org
    ).delete_undeployed_revisions(name, save_last=save_last, dry_run=dry_run)


@apis.command(
    help='Deletes all undeployed revisions of an API proxy and all policies, resources, endpoints, and revisions associated with it.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option(
    '--save-last',
    type=click.INT,
    help='denotes not to delete the N most recent revisions',
    default=0,
)
@click.option(
    '--dry-run/--no-dry-run', default=False, help='show revisions to be deleted but do not delete'
)
def clean(*args, **kwargs):
    _delete_undeployed_revisions(*args, **kwargs)


def _export_api_proxy(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    revision_number,
    fs_write=True,
    output_file=None,
    **kwargs,
):
    return Apis(gen_auth(username, password, mfa_secret, token, zonename), org).export_api_proxy(
        name,
        revision_number,
        fs_write=True,
        output_file=output_file if output_file else f'{name}.zip',
    )


@apis.command(
    help='Outputs an API proxy revision as a ZIP formatted bundle of code and config files. This enables local configuration and development, including attachment of policies and scripts.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
@click.option(
    '-O', '--output-file', help='specify output file (defaults to API_NAME.zip)', default=None
)
def export(*args, **kwargs):
    _export_api_proxy(*args, **kwargs)


def _get_api_proxy(username, password, mfa_secret, token, zonename, org, profile, name, **kwargs):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_api_proxy(name)
        .text
    )


@apis.command(
    help='Gets an API proxy by name, including a list of existing revisions of the proxy.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
def get(*args, **kwargs):
    console.echo(_get_api_proxy(*args, **kwargs))


def _list_api_proxies(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    prefix=None,
    format='json',
    **kwargs,
):
    return Apis(gen_auth(username, password, mfa_secret, token, zonename), org).list_api_proxies(
        prefix=prefix
    )


@apis.command(
    help='Gets the names of all API proxies in an organization. The names correspond to the names defined in the configuration files for each API proxy.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@common_prefix_options
def list(*args, **kwargs):
    console.echo(_list_api_proxies(*args, **kwargs))


def _list_api_proxy_revisions(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .list_api_proxy_revisions(name)
        .text
    )


@apis.command(help='List all revisions for an API proxy.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
def list_revisions(*args, **kwargs):
    console.echo(_list_api_proxy_revisions(*args, **kwargs))


def _undeploy_api_proxy_revision(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    revision_number,
    **kwargs,
):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .undeploy_api_proxy_revision(name, environment, revision_number)
        .text
    )


@apis.command(help='Undeploys an API proxy revision from an environment.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
def undeploy_revision(*args, **kwargs):
    console.echo(_undeploy_api_proxy_revision(*args, **kwargs))


def _force_undeploy_api_proxy_revision(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    revision_number,
    **kwargs,
):
    return (
        Apis(gen_auth(username, password, mfa_secret, token, zonename), org)
        .force_undeploy_api_proxy_revision(name, environment, revision_number)
        .text
    )


@apis.command(help='Force the undeployment of the API proxy that is partially deployed.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
def force_undeploy_revision(*args, **kwargs):
    console.echo(_force_undeploy_api_proxy_revision(*args, **kwargs))


def _pull(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    revision_number,
    environment,
    work_tree,
    dependencies=[],
    force=False,
    prefix=None,
    basepath=None,
    **kwargs,
):
    return Apis(
        gen_auth(username, password, mfa_secret, token, zonename),
        org,
        revision_number,
        environment,
        work_tree=work_tree,
    ).pull(name, force=force, prefix=prefix, basepath=basepath)


@apis.command(
    help='Downloads an API proxy revision, along with any referenced key/value maps, target servers and caches into the current working directory.'
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option(
    '--work-tree', help='set the path to the working tree (defaults to current working directory)'
)
@click.option('--force/--no-force', '-f/-F', default=False, help='force write files')
@click.option(
    '--prefix',
    help='prefix to prepend to names. WARNING: this is not foolproof. make sure to review the changes.',
    hidden=True,
)
@click.option(
    '-b', '--basepath', help='set default basepath in apiproxy/proxies/default.xml', hidden=True
)
def pull(*args, **kwargs):
    _pull(*args, **kwargs)


def _deploy(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    directory,
    import_only,
    seamless_deploy,
    environment,
    **kwargs,
):
    return deploy_tool(
        Struct(
            username=username,
            password=password,
            directory=directory,
            org=org,
            name=name,
            environment=environment,
            import_only=import_only,
            seamless_deploy=seamless_deploy,
            mfa_secret=mfa_secret,
            token=token,
            zonename=zonename,
        )
    )


@apis.command(
    help="""Deploy APIs using an improved version of the Apigee API Proxy Deploy Tool: https://github.com/apigee/api-platform-samples/tree/master/tools

\b
   =========================================================================
   ==  NOTICE file corresponding to the section 4 d of                    ==
   ==  the Apache License, Version 2.0,                                   ==
   ==  in this case for the Apigee API Proxy Deploy Tool code.            ==
   =========================================================================

Apigee API Proxy Deploy Tool
https://github.com/apigee/api-platform-samples/tree/master/tools
These files are Copyright 2015 Apigee Corporation, released under the Apache2 License.
"""
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-n', '--name', help='name', required=True)
@click.option(
    '-d',
    '--directory',
    help='directory with the apiproxy/ bundle',
    type=click.Path(exists=True, dir_okay=True, file_okay=False, resolve_path=False),
    required=True,
)
@optgroup.group(
    'Deployment options', cls=MutuallyExclusiveOptionGroup, help='The deployment options'
)
@optgroup.option(
    '--import-only/--no-import-only', '-i/-I', default=False, help='import only and not deploy'
)
@optgroup.option(
    '--seamless-deploy/--no-seamless-deploy',
    '-s/-S',
    default=False,
    help='seamless deploy the bundle',
)
def deploy(*args, **kwargs):
    _deploy(*args, **kwargs)



import json

import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.apiproducts.serializer import ApiproductsSerializer
from apigee.utils import read_file

CREATE_API_PRODUCT_PATH = '{api_url}/v1/organizations/{org}/apiproducts'
DELETE_API_PRODUCT_PATH = '{api_url}/v1/organizations/{org}/apiproducts/{name}'
GET_API_PRODUCT_PATH = '{api_url}/v1/organizations/{org}/apiproducts/{name}'
LIST_API_PRODUCTS_PATH = (
    '{api_url}/v1/organizations/{org}/apiproducts?expand={expand}&count={count}&startKey={startkey}'
)
UPDATE_API_PRODUCT_PATH = '{api_url}/v1/organizations/{org}/apiproducts/{name}'


class Apiproducts:
    def __init__(self, auth, org_name, apiproduct_name):
        self._auth = auth
        self._org_name = org_name
        self._apiproduct_name = apiproduct_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def apiproduct_name(self):
        return self._apiproduct_name

    @apiproduct_name.setter
    def apiproduct_name(self, value):
        self._apiproduct_name = value

    def create_api_product(self, request_body):
        uri = CREATE_API_PRODUCT_PATH.format(api_url=APIGEE_ADMIN_API_URL, org=self._org_name)
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_api_product(self):
        uri = DELETE_API_PRODUCT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, name=self._apiproduct_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_api_product(self):
        uri = GET_API_PRODUCT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, name=self._apiproduct_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_api_products(self, prefix=None, expand=False, count=1000, startkey="", format='json'):
        uri = LIST_API_PRODUCTS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            expand=expand,
            count=count,
            startkey=startkey,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return ApiproductsSerializer().serialize_details(resp, format, prefix=prefix)

    def update_api_product(self, request_body):
        uri = UPDATE_API_PRODUCT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, name=self._apiproduct_name
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.put(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def push_apiproducts(self, file):
        apiproduct = read_file(file, type='json')
        self._apiproduct_name = apiproduct['name']
        try:
            self.get_api_product()
            console.echo(f'Updating {self._apiproduct_name}')
            console.echo(self.update_api_product(json.dumps(apiproduct)).text)
        except HTTPError as e:
            if e.response.status_code != 404:
                raise e
            console.echo(f'Creating {self._apiproduct_name}')
            console.echo(self.create_api_product(json.dumps(apiproduct)).text)



import json

import requests

from apigee import APIGEE_ADMIN_API_URL, auth
from apigee.permissions.serializer import PermissionsSerializer

CREATE_PERMISSIONS_PATH = (
    '{api_url}/v1/organizations/{org}/userroles/{role_name}/resourcepermissions'
)
TEAM_PERMISSIONS_PATH = '{api_url}/v1/organizations/{org}/userroles/{role_name}/resourcepermissions'
GET_PERMISSIONS_PATH = '{api_url}/v1/o/{org}/userroles/{role_name}/permissions'


class Permissions:
    def __init__(self, auth, org_name, role_name):
        self._auth = auth
        self._org_name = org_name
        self._role_name = role_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def role_name(self):
        return self._role_name

    @role_name.setter
    def role_name(self, value):
        self._role_name = value

    def create_permissions(self, request_body):
        uri = CREATE_PERMISSIONS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, role_name=self._role_name
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def team_permissions(self, template_file, placeholder_key=None, placeholder_value=""):
        uri = TEAM_PERMISSIONS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, role_name=self._role_name
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        with open(template_file) as f:
            body = json.loads(f.read())
        if placeholder_key:
            for idx, resource_permission in enumerate(body['resourcePermission']):
                path = resource_permission['path']
                body['resourcePermission'][idx]['path'] = path.replace(
                    placeholder_key, placeholder_value
                )
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def get_permissions(self, formatted=False, format='text', showindex=False, tablefmt='plain'):
        uri = GET_PERMISSIONS_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, role_name=self._role_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        if formatted:
            return PermissionsSerializer().serialize_details(
                resp, format, showindex=showindex, tablefmt=tablefmt
            )
        return resp



import json

import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.virtualhosts.serializer import VirtualhostsSerializer

CREATE_A_VIRTUAL_HOST_FOR_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/virtualhosts'
)
DELETE_A_VIRTUAL_HOST_FROM_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/virtualhosts/{virtualhost_name}'
)
GET_A_VIRTUAL_HOST_FOR_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/virtualhosts/{virtualhost_name}'
)
LIST_VIRTUAL_HOSTS_FOR_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/virtualhosts'
)
UPDATE_VIRTUAL_HOST_FOR_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/o/{org_name}/environments/{environment}/virtualhosts/{virtualhost_name}'
)


class Virtualhosts:
    def __init__(self, auth, org_name, virtualhost_name):
        self._auth = auth
        self._org_name = org_name
        self._virtualhost_name = virtualhost_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def virtualhost_name(self):
        return self._virtualhost_name

    @virtualhost_name.setter
    def virtualhost_name(self, value):
        self._virtualhost_name = value

    def create_a_virtual_host_for_an_environment(self):
        pass

    def delete_a_virtual_host_from_an_environment(self):
        pass

    def get_a_virtual_host_for_an_environment(self, environment):
        uri = GET_A_VIRTUAL_HOST_FOR_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            virtualhost_name=self._virtualhost_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_virtual_hosts_for_an_environment(self, environment, prefix=None, format='json'):
        uri = LIST_VIRTUAL_HOSTS_FOR_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org_name=self._org_name, environment=environment
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return VirtualhostsSerializer().serialize_details(resp, format, prefix=prefix)

    def update_virtual_host_for_an_environment(self):
        pass



import json

import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.maskconfigs.serializer import MaskconfigsSerializer
from apigee.utils import read_file

CREATE_DATA_MASKS_FOR_AN_API_PROXY_PATH = (
    '{api_url}/v1/organizations/{org}/apis/{api_name}/maskconfigs'
)
DELETE_DATA_MASKS_FOR_AN_API_PROXY_PATH = (
    '{api_url}/v1/organizations/{org}/apis/{api_name}/maskconfigs/{maskconfig_name}'
)
GET_DATA_MASK_DETAILS_FOR_AN_API_PROXY_PATH = (
    '{api_url}/v1/organizations/{org}/apis/{api_name}/maskconfigs/{maskconfig_name}'
)
LIST_DATA_MASKS_FOR_AN_API_PROXY_PATH = (
    '{api_url}/v1/organizations/{org}/apis/{api_name}/maskconfigs'
)
LIST_DATA_MASKS_FOR_AN_ORGANIZATION_PATH = '{api_url}/v1/organizations/{org}/maskconfigs'


class Maskconfigs:
    def __init__(self, auth, org_name, api_name):
        self._auth = auth
        self._org_name = org_name
        self._api_name = api_name

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def api_name(self):
        return self._api_name

    @api_name.setter
    def api_name(self, value):
        self._api_name = value

    def __call__(self):
        pass

    def create_data_masks_for_an_api_proxy(self, request_body):
        uri = CREATE_DATA_MASKS_FOR_AN_API_PROXY_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, api_name=self._api_name
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_data_masks_for_an_api_proxy(self, maskconfig_name):
        uri = DELETE_DATA_MASKS_FOR_AN_API_PROXY_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            api_name=self._api_name,
            maskconfig_name=maskconfig_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_data_mask_details_for_an_api_proxy(self, maskconfig_name):
        uri = GET_DATA_MASK_DETAILS_FOR_AN_API_PROXY_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            api_name=self._api_name,
            maskconfig_name=maskconfig_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_data_masks_for_an_api_proxy(self):
        uri = LIST_DATA_MASKS_FOR_AN_API_PROXY_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, api_name=self._api_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_data_masks_for_an_organization(self):
        uri = LIST_DATA_MASKS_FOR_AN_ORGANIZATION_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def push_data_masks_for_an_api_proxy(self, file):
        maskconfig = read_file(file, type='json')
        maskconfig_name = maskconfig['name']
        try:
            self.get_data_mask_details_for_an_api_proxy(maskconfig_name)
            console.echo(f'Updating {maskconfig_name} for {self._api_name}')
            console.echo(self.create_data_masks_for_an_api_proxy(json.dumps(maskconfig)).text)
        except HTTPError as e:
            if e.response.status_code != 404:
                raise e
            console.echo(f'Creating {maskconfig_name} for {self._api_name}')
            console.echo(self.create_data_masks_for_an_api_proxy(json.dumps(maskconfig)).text)



import json


class KeystoresSerializer:
    def serialize_details(self, keystores, format, prefix=None):
        resp = keystores
        if format == 'text':
            return keystores.text
        keystores = keystores.json()
        if prefix:
            keystores = [keystore for keystore in keystores if keystore.startswith(prefix)]
        if format == 'json':
            return json.dumps(keystores)
        elif format == 'table':
            pass
        elif format == 'dict':
            return keystores
        return resp



import json


class TargetserversSerializer:
    def serialize_details(self, targetservers, format, prefix=None):
        resp = targetservers
        if format == 'text':
            return targetservers.text
        targetservers = targetservers.json()
        if prefix:
            targetservers = [
                targetserver for targetserver in targetservers if targetserver.startswith(prefix)
            ]
        if format == 'json':
            return json.dumps(targetservers)
        elif format == 'table':
            pass
        elif format == 'dict':
            return targetservers
        return resp



import builtins
import sys


def echo(
    *message, status=None, silent=False, curr_verbosity=0, expc_verbosity=0, end='\n', flush=False
):
    toggle_silent = builtins.APIGEE_CLI_TOGGLE_SILENT
    toggle_verbose = builtins.APIGEE_CLI_TOGGLE_VERBOSE
    if silent or toggle_silent:
        if status:
            sys.exit(status)
        return
    if curr_verbosity or toggle_verbose >= expc_verbosity:
        print(*message, end=end, flush=flush)
    if status:
        sys.exit(status)



import click
from click_option_group import RequiredMutuallyExclusiveOptionGroup, optgroup

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.prefix import common_prefix_options
from apigee.sharedflows.sharedflows import Sharedflows
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options


@click.group(help='You can use the following APIs to manage shared flows and flow hooks.')
def sharedflows():
    pass


def _get_a_list_of_shared_flows(
    username, password, mfa_secret, token, zonename, org, profile, prefix=None, **kwargs
):
    return Sharedflows(
        gen_auth(username, password, mfa_secret, token, zonename), org
    ).get_a_list_of_shared_flows(prefix=prefix)


@sharedflows.command(
    help='Gets an array of the names of shared flows in the organization. The response is a simple array of strings.'
)
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
def list(*args, **kwargs):
    console.echo(_get_a_list_of_shared_flows(*args, **kwargs))


def _import_a_shared_flow(
    username, password, mfa_secret, token, zonename, org, profile, file, name, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .import_a_shared_flow(file, name)
        .text
    )


@sharedflows.command(
    name='import',
    help='Uploads a ZIP-formatted shared flow configuration bundle from a local machine to an Edge organization. If the shared flow already exists, this creates a new revision of it. If the shared flow does not exist, this creates it. Once imported, the shared flow revision must be deployed before it can be accessed at runtime. By default, shared flow configurations are not validated on import.',
)
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
    help='file path of the shared flow configuration in ZIP format',
)
@click.option('-n', '--name', help='name', required=True)
def import_a_shared_flow(*args, **kwargs):
    console.echo(_import_a_shared_flow(*args, **kwargs))


# def _export_a_shared_flow(self, shared_flow_name, revision_number):
#     pass


def _get_a_shared_flow(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_a_shared_flow(name)
        .text
    )


@sharedflows.command(help='Gets a shared flow by name, including a list of its revisions.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
def get(*args, **kwargs):
    console.echo(_get_a_shared_flow(*args, **kwargs))


def _deploy_a_shared_flow(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    name,
    revision_number,
    override,
    delay,
    file,
    **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .deploy_a_shared_flow(
            environment,
            name,
            revision_number,
            override=override,
            delay=delay,
            shared_flow_file=file,
        )
        .text
    )


@sharedflows.command(
    help='Deploys a shared flow revision to an environment in an organization. Shared flows cannot be used until they have been deployed to an environment. If you experience HTTP 500 errors during deployment, consider using the override parameter to deploy the shared flow in place of a revision currently deployed. The size limit of a shared flow bundle is 15 MB.'
)
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-n', '--name', help='name', required=True)
@optgroup.group(
    'Deployment options', cls=RequiredMutuallyExclusiveOptionGroup, help='The deployment options'
)
@optgroup.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    help='file path of the shared flow configuration in ZIP format',
)
@optgroup.option('-r', '--revision-number', type=click.INT, help='revision number')
@click.option(
    '--override/--no-override',
    default=False,
    show_default=True,
    help='Set this flag to "true" to deploy the shared flow revision in place of the revision currently deployed (also known as "zero downtime" deployment).',
)
@click.option(
    '--delay',
    type=click.INT,
    default=0,
    show_default=True,
    help='Enforces a delay, measured in seconds, before the currently deployed API proxy revision is undeployed and replaced by the new revision that is being deployed. Use this setting to minimize the impact of deployment on in-flight transactions. The default value is 0.',
)
def deploy(*args, **kwargs):
    console.echo(_deploy_a_shared_flow(*args, **kwargs))


def _undeploy_a_shared_flow(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    name,
    revision_number,
    **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .undeploy_a_shared_flow(environment, name, revision_number)
        .text
    )


@sharedflows.command(help='Undeploys a shared flow revision from an environment.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-n', '--name', help='name', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
def undeploy(*args, **kwargs):
    console.echo(_undeploy_a_shared_flow(*args, **kwargs))


def _get_deployment_environments_for_shared_flows(
    username, password, mfa_secret, token, zonename, org, profile, name, revision_number, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_deployment_environments_for_shared_flows(name, revision_number)
        .text
    )


@sharedflows.command(help='Gets an array of the environments to which the shared flow is deployed.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-r', '--revision-number', type=click.INT, help='revision number', required=True)
def get_deployment_environments(*args, **kwargs):
    console.echo(_get_deployment_environments_for_shared_flows(*args, **kwargs))


def _delete_a_shared_flow(self, shared_flow_name):
    pass


# def _attach_a_shared_flow_to_a_flow_hook(self, environment, flow_hook, request_body):
#     pass
#
# def _detaches_a_shared_flow_from_a_flow_hook(self, environment, flow_hook):
#     pass


def _get_the_shared_flow_attached_to_a_flow_hook(
    username, password, mfa_secret, token, zonename, org, profile, environment, flow_hook, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_the_shared_flow_attached_to_a_flow_hook(environment, flow_hook)
        .text
    )


@sharedflows.command(
    help="Returns the name of the shared flow attached to the specified flow hook. If there's no shared flow attached to the flow hook, the API does not return an error; it simply does not return a name in the response. Specify one of these flowhook locations in the API: PreProxyFlowHook, PreTargetFlowHook, PostTargetFlowHook, PostProxyFlowHook."
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option('--flow-hook', help='flow hook to which the shared flow is attached', required=True)
def get_flow_hook(*args, **kwargs):
    console.echo(_get_the_shared_flow_attached_to_a_flow_hook(*args, **kwargs))


def _get_shared_flow_deployments(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_shared_flow_deployments(name)
        .text
    )


@sharedflows.command(help='View all shared flow deployments.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
def get_deployments(*args, **kwargs):
    console.echo(_get_shared_flow_deployments(*args, **kwargs))


def _get_shared_flow_revisions(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Sharedflows(gen_auth(username, password, mfa_secret, token, zonename), org)
        .get_shared_flow_revisions(name)
        .text
    )


@sharedflows.command(help='List the revisions of a shared flow.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
def get_revisions(*args, **kwargs):
    console.echo(_get_shared_flow_revisions(*args, **kwargs))



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.maskconfigs.maskconfigs import Maskconfigs
# from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options


@click.group(help='Specify data that will be filtered out of trace sessions.')
def maskconfigs():
    pass


def _create_data_masks_for_an_api_proxy(
    username, password, mfa_secret, token, zonename, org, profile, name, body, **kwargs
):
    return (
        Maskconfigs(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_data_masks_for_an_api_proxy(body)
        .text
    )


@maskconfigs.command(
    help="Create a data mask for an API proxy. You can capture message content to assist in runtime debugging of APIs calls. In many cases, API traffic contains sensitive data, such credit cards or personally identifiable health information (PHI) that needs to filtered out of the captured message content. Data masks enable you to specify data that will be filtered out of trace sessions. Data masking is only enabled when a trace session (also called a 'debug' session) is enabled for an API proxy. If no trace session are enabled on an API proxy, then the data will not be masked."
)
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='API proxy name', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create_api(*args, **kwargs):
    console.echo(_create_data_masks_for_an_api_proxy(*args, **kwargs))


def _delete_data_masks_for_an_api_proxy(
    username, password, mfa_secret, token, zonename, org, profile, name, maskconfig_name, **kwargs
):
    return (
        Maskconfigs(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_data_masks_for_an_api_proxy(maskconfig_name)
        .text
    )


@maskconfigs.command(help='Delete a data mask for an API proxy.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='API proxy name', required=True)
@click.option('--maskconfig-name', help='data mask name', required=True)
def delete_api(*args, **kwargs):
    console.echo(_delete_data_masks_for_an_api_proxy(*args, **kwargs))


def _get_data_mask_details_for_an_api_proxy(
    username, password, mfa_secret, token, zonename, org, profile, name, maskconfig_name, **kwargs
):
    return (
        Maskconfigs(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_data_mask_details_for_an_api_proxy(maskconfig_name)
        .text
    )


@maskconfigs.command(help='Get the details for a data mask for an API proxy.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='API proxy name', required=True)
@click.option('--maskconfig-name', help='data mask name', required=True)
def get_api(*args, **kwargs):
    console.echo(_get_data_mask_details_for_an_api_proxy(*args, **kwargs))


def _list_data_masks_for_an_api_proxy(
    username, password, mfa_secret, token, zonename, org, profile, name, **kwargs
):
    return (
        Maskconfigs(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .list_data_masks_for_an_api_proxy()
        .text
    )


@maskconfigs.command(help='List all data masks for an API proxy.')
@common_auth_options
@common_verbose_options
@common_silent_options
@click.option('-n', '--name', help='API proxy name', required=True)
def list_api(*args, **kwargs):
    console.echo(_list_data_masks_for_an_api_proxy(*args, **kwargs))


def _list_data_masks_for_an_organization(
    username, password, mfa_secret, token, zonename, org, profile, **kwargs
):
    return (
        Maskconfigs(gen_auth(username, password, mfa_secret, token, zonename), org, None)
        .list_data_masks_for_an_organization()
        .text
    )


@maskconfigs.command(help='List all data masks for an organization.')
@common_auth_options
@common_verbose_options
@common_silent_options
def list(*args, **kwargs):
    console.echo(_list_data_masks_for_an_organization(*args, **kwargs))


def _push_data_masks_for_an_api_proxy(
    username, password, mfa_secret, token, zonename, org, profile, name, file, **kwargs
):
    return Maskconfigs(
        gen_auth(username, password, mfa_secret, token, zonename), org, name
    ).push_data_masks_for_an_api_proxy(file)


@maskconfigs.command(
    help='Push data mask for an API proxy to Apigee. This will create/update a data mask.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='API proxy name', required=True)
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
def push(*args, **kwargs):
    _push_data_masks_for_an_api_proxy(*args, **kwargs)



import json

import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.references.serializer import ReferencesSerializer

LIST_ALL_REFERENCES_PATH = (
    '{api_url}/v1/organizations/{org_name}/environments/{environment}/references'
)
GET_REFERENCE_PATH = (
    '{api_url}/v1/organizations/{org_name}/environments/{environment}/references/{ref_name}'
)
DELETE_REFERENCE_PATH = (
    '{api_url}/v1/organizations/{org_name}/environments/{environment}/references/{ref_name}'
)
CREATE_REFERENCE_PATH = (
    '{api_url}/v1/organizations/{org_name}/environments/{environment}/references'
)
UPDATE_REFERENCE_PATH = (
    '{api_url}/v1/organizations/{org_name}/environments/{environment}/references/{ref_name}'
)


class References:
    def __init__(self, auth, org_name, ref_name):
        self._auth = auth
        self._org_name = org_name
        self._ref_name = ref_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def ref_name(self):
        return self._ref_name

    @ref_name.setter
    def ref_name(self, value):
        self._ref_name = value

    def list_all_references(self, environment, prefix=None, format='json'):
        uri = LIST_ALL_REFERENCES_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org_name=self._org_name, environment=environment
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return ReferencesSerializer().serialize_details(resp, format, prefix=prefix)

    def get_reference(self, environment):
        uri = GET_REFERENCE_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org_name=self._org_name,
            environment=environment,
            ref_name=self._ref_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_reference(self):
        pass

    def create_reference(self):
        pass

    def update_reference(self):
        pass



import json


class ApiproductsSerializer:
    def serialize_details(self, apiproducts, format, prefix=None):
        resp = apiproducts
        if format == 'text':
            return apiproducts.text
        apiproducts = apiproducts.json()
        if prefix:
            apiproducts = [
                apiproduct for apiproduct in apiproducts if apiproduct.startswith(prefix)
            ]
        if format == 'json':
            return json.dumps(apiproducts)
        elif format == 'table':
            pass
        elif format == 'dict':
            return apiproducts
        return resp



import configparser

import click

from apigee import (APIGEE_CLI_AUTHORIZATION_DEVELOPER_ATTRIBUTE,
                    APIGEE_CLI_CREDENTIALS_FILE)
from apigee.developers.developers import Developers


def common_prefix_options(func):
    config = configparser.ConfigParser()
    config.read(APIGEE_CLI_CREDENTIALS_FILE)
    profile = 'default'
    import sys

    for i, arg in enumerate(sys.argv):
        if arg == '-P' or arg == '--profile':
            try:
                profile = sys.argv[i + 1]
            except IndexError:
                pass
    profile_dict = {}
    try:
        profile_dict = dict(config._sections[profile])
    except KeyError:
        pass
    prefix = ''
    try:
        prefix = profile_dict['prefix']
    except KeyError:
        pass
    return click.option(
        '--prefix', help='team/resource prefix filter', default=prefix, show_default=True
    )(func)


def auth_with_prefix(
    auth_obj, org, name, attribute_name=APIGEE_CLI_AUTHORIZATION_DEVELOPER_ATTRIBUTE
):
    team = (
        Developers(auth_obj, org, auth_obj.username)
        .get_developer_attribute(attribute_name)
        .json()['value']
    )
    allowed = team.split(',')
    for prefix in allowed:
        if name.startswith(prefix):
            return name
    raise Exception(
        f'401 Client Error: Unauthorized for team: {str(allowed)}\nAttempted to access resource: {name}'
    )



import json


class MaskconfigsSerializer:
    def serialize_details(self, maskconfigs, format, prefix=None):
        resp = maskconfigs
        if format == 'text':
            return maskconfigs.text
        maskconfigs = maskconfigs.json()
        if prefix:
            maskconfigs = [
                maskconfig for maskconfig in maskconfigs if maskconfig.startswith(prefix)
            ]
        if format == 'json':
            return json.dumps(maskconfigs)
        elif format == 'table':
            pass
        return resp



import json

from tabulate import tabulate


class DeploymentsSerializer:
    def serialize_details(self, deployment_details, format, showindex=False, tablefmt='plain'):
        if format == 'text':
            return deployment_details.text
        revisions = []
        for i in deployment_details.json()['environment']:
            revisions.append(
                {
                    'name': i['name'],
                    'revision': [j['name'] for j in i['revision']],
                    'state': [j['state'] for j in i['revision']],
                }
            )
        if format == 'json':
            return json.dumps(revisions)
        elif format == 'table':
            table = [[rev['name'], rev['revision'], rev['state']] for rev in revisions]
            headers = []
            if showindex == 'always' or showindex is True:
                headers = ['id', 'name', 'revision', 'state']
            elif showindex == 'never' or showindex is False:
                headers = ['name', 'revision', 'state']
            return tabulate(table, headers, showindex=showindex, tablefmt=tablefmt)
        else:
            raise ValueError(format)
        return deployment_details



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.targetservers.targetservers import Targetservers
from apigee.verbose import common_verbose_options


@click.group(
    help='TargetServers are used to decouple TargetEndpoint HTTPTargetConnections from concrete URLs for backend services.'
)
def targetservers():
    pass


def _create_a_targetserver(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Targetservers(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .create_a_targetserver(environment, body)
        .text
    )


@targetservers.command(
    help='Create a TargetServer in the specified environment. TargetServers are used to decouple TargetEndpoint HTTPTargetConnections from concrete URLs for backend services.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create(*args, **kwargs):
    console.echo(_create_a_targetserver(*args, **kwargs))


def _delete_a_targetserver(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Targetservers(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .delete_a_targetserver(environment)
        .text
    )


@targetservers.command(
    help='Delete a TargetServer configuration from an environment. Returns information about the deleted TargetServer.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def delete(*args, **kwargs):
    console.echo(_delete_a_targetserver(*args, **kwargs))


def _list_targetservers_in_an_environment(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    prefix=None,
    **kwargs
):
    return Targetservers(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).list_targetservers_in_an_environment(environment, prefix=prefix)


@targetservers.command(help='List all TargetServers in an environment.')
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
def list(*args, **kwargs):
    console.echo(_list_targetservers_in_an_environment(*args, **kwargs))


def _get_targetserver(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Targetservers(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_targetserver(environment)
        .text
    )


@targetservers.command(help='Returns a TargetServer definition.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get(*args, **kwargs):
    console.echo(_get_targetserver(*args, **kwargs))


def _update_a_targetserver(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, body, **kwargs
):
    return (
        Targetservers(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .update_a_targetserver(environment, body)
        .text
    )


@targetservers.command(help='Modifies an existing TargetServer.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
@click.option('-b', '--body', help='request body', required=True)
def create(*args, **kwargs):
    console.echo(_update_a_targetserver(*args, **kwargs))


def _push_targetserver(
    username, password, mfa_secret, token, zonename, org, profile, environment, file, **kwargs
):
    return Targetservers(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).push_targetserver(environment, file)


@targetservers.command(help='Push TargetServer to Apigee. This will create/update a TargetServer.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
@click.option(
    '-f',
    '--file',
    type=click.Path(exists=True, dir_okay=False, file_okay=True, resolve_path=False),
    required=True,
)
def push(*args, **kwargs):
    _push_targetserver(*args, **kwargs)



import json


class CachesSerializer:
    def serialize_details(self, caches, format, prefix=None):
        resp = caches
        if format == 'text':
            return caches.text
        caches = caches.json()
        if prefix:
            caches = [cache for cache in caches if cache.startswith(prefix)]
        if format == 'json':
            return json.dumps(caches)
        elif format == 'table':
            pass
        elif format == 'dict':
            return caches
        return resp



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.backups.backups import Backups
# from apigee.cls import OptionEatAll
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options

APIS_CHOICES = {
    'apis',
    'keyvaluemaps',
    'targetservers',
    'caches',
    'developers',
    'apiproducts',
    'apps',
    'userroles',
}


@click.group(help='Download configuration files from Apigee that can later be restored.')
def backups():
    pass


def _take_snapshot(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    target_directory,
    prefix,
    environments,
    apis,
    **kwargs
):
    if not isinstance(apis, set):
        apis = set(apis)
    Backups(
        gen_auth(username, password, mfa_secret, token, zonename),
        org,
        target_directory,
        prefix=prefix,
        fs_write=True,
        apis=apis,
        environments=list(environments),
    ).take_snapshot()


@backups.command(
    help='Downloads and generates local snapshots of specified Apigee resources e.g. API proxies, KVMs, target servers, etc.'
)
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option(
    '--target-directory',
    type=click.Path(exists=False, dir_okay=True, file_okay=False, resolve_path=False),
    required=True,
)
@click.option(
    '--apis',
    type=click.Choice(APIS_CHOICES, case_sensitive=False),
    multiple=True,
    default=APIS_CHOICES,
    show_default=True,
)
# @click.option('--apis', metavar='LIST', cls=OptionEatAll, default=APIS_CHOICES, show_default=True, help='')
# @click.option(
#     '-e', '--environments', metavar='LIST', cls=OptionEatAll, default=['test', 'prod'], help=''
# )
@click.option(
    '-e', '--environments', multiple=True, show_default=True, default=['test', 'prod'], help=''
)
def take_snapshot(*args, **kwargs):
    _take_snapshot(*args, **kwargs)



import click
from click_aliases import ClickAliasedGroup

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.deployments.deployments import Deployments
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options

TABLEFMT_CHOICES = [
    'plain',
    'simple',
    'github',
    'grid',
    'fancy_grid',
    'pipe',
    'orgtbl',
    'jira',
    'presto',
    'psql',
    'rst',
    'mediawiki',
    'moinmoin',
    'youtrack',
    'html',
    'latex',
    'latex_raw',
    'latex_booktabs',
    'textile',
]


@click.group(
    help='API proxies that are actively deployed in environments on Apigee Edge.',
    cls=ClickAliasedGroup,
)
def deployments():
    pass


def _get_api_proxy_deployment_details(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    format,
    showindex,
    tablefmt,
    revision_name_only,
    **kwargs
):
    return Deployments(
        gen_auth(username, password, mfa_secret, token, zonename), org, name
    ).get_api_proxy_deployment_details(
        formatted=True,
        format=format,
        showindex=showindex,
        tablefmt=tablefmt,
        revision_name_only=revision_name_only,
    )


@deployments.command(
    help='Returns detail on all deployments of the API proxy for all environments. All deployments are listed in the test and prod environments, as well as other environments, if they exist.',
    aliases=['get-api-proxy-deployment-details'],
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='name', required=True)
# @click.option("-j", "--json", help="display json output when using -r flag", default="table")
@click.option(
    '--format',
    help='defines how to format output when using the -r flag',
    default='table',
    type=click.Choice(['json', 'table'], case_sensitive=False),
)
@click.option('--showindex/--no-showindex', default=False)
@click.option(
    '--tablefmt',
    help='defines how the table is formatted',
    type=click.Choice(TABLEFMT_CHOICES, case_sensitive=False),
    default='plain',
    show_default=True,
)
@click.option('--revision-name-only/--no-revision-name-only', '-r/-R', default=False)
def get(*args, **kwargs):
    console.echo(_get_api_proxy_deployment_details(*args, **kwargs))



import json

import requests
from requests.exceptions import HTTPError
from tqdm import tqdm

from apigee import APIGEE_ADMIN_API_URL, auth, console
from apigee.keyvaluemaps.serializer import KeyvaluemapsSerializer
from apigee.utils import read_file

CREATE_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps'
)
DELETE_KEYVALUEMAP_FROM_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}'
)
DELETE_KEYVALUEMAP_ENTRY_IN_AN_ENVIRONMENT_PATH = '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}/entries/{entry_name}'
GET_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}'
)
GET_A_KEYS_VALUE_IN_AN_ENVIRONMENT_SCOPED_KEYVALUEMAP_PATH = '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}/entries/{entry_name}'
LIST_KEYVALUEMAPS_IN_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps'
)
UPDATE_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}'
)
CREATE_AN_ENTRY_IN_AN_ENVIRONMENT_SCOPED_KVM_PATH = (
    '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}/entries'
)
UPDATE_AN_ENTRY_IN_AN_ENVIRONMENT_SCOPED_KVM_PATH = '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}/entries/{entry_name}'
LIST_KEYS_IN_AN_ENVIRONMENT_SCOPED_KEYVALUEMAP_PATH = '{api_url}/v1/organizations/{org}/environments/{environment}/keyvaluemaps/{name}/keys?startkey={startkey}&count={count}'


def TQDM_KWARGS(desc):
    return {
        'desc': desc,
        'unit': 'entries',
        'bar_format': '{l_bar}{bar:32}{r_bar}{bar:-10b}',
        'leave': False,
    }


class Keyvaluemaps:
    def __init__(self, auth, org_name, map_name):
        self._auth = auth
        self._org_name = org_name
        self._map_name = map_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def map_name(self):
        return self._map_name

    @map_name.setter
    def map_name(self, value):
        self._map_name = value

    def __create_or_update_entry(self, environment, entry):
        try:
            self.get_a_keys_value_in_an_environment_scoped_keyvaluemap(environment, entry['name'])
            self.update_an_entry_in_an_environment_scoped_kvm(
                environment, entry['name'], entry['value']
            )
        except HTTPError as e:
            if e.response.status_code != 404:
                raise e
            self.create_an_entry_in_an_environment_scoped_kvm(
                environment, entry['name'], entry['value']
            )

    def __delete_entries(self, environment, deleted_keys):
        for idx, entry in enumerate(tqdm(deleted_keys, **TQDM_KWARGS('Deleting'))):
            self.delete_keyvaluemap_entry_in_an_environment(environment, entry['name'])

    def create_keyvaluemap_in_an_environment(self, environment, request_body):
        uri = CREATE_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, environment=environment
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_keyvaluemap_from_an_environment(self, environment):
        uri = DELETE_KEYVALUEMAP_FROM_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_keyvaluemap_entry_in_an_environment(self, environment, entry_name):
        uri = DELETE_KEYVALUEMAP_ENTRY_IN_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
            entry_name=entry_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_keyvaluemap_in_an_environment(self, environment):
        uri = GET_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_a_keys_value_in_an_environment_scoped_keyvaluemap(self, environment, entry_name):
        uri = GET_A_KEYS_VALUE_IN_AN_ENVIRONMENT_SCOPED_KEYVALUEMAP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
            entry_name=entry_name,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_keyvaluemaps_in_an_environment(self, environment, prefix=None, format='json'):
        uri = LIST_KEYVALUEMAPS_IN_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL, org=self._org_name, environment=environment
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return KeyvaluemapsSerializer().serialize_details(resp, format, prefix=prefix)

    def update_keyvaluemap_in_an_environment(self, environment, request_body):
        uri = UPDATE_KEYVALUEMAP_IN_AN_ENVIRONMENT_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def create_an_entry_in_an_environment_scoped_kvm(self, environment, entry_name, entry_value):
        uri = CREATE_AN_ENTRY_IN_AN_ENVIRONMENT_SCOPED_KVM_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = {'name': entry_name, 'value': entry_value}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def update_an_entry_in_an_environment_scoped_kvm(self, environment, entry_name, updated_value):
        uri = UPDATE_AN_ENTRY_IN_AN_ENVIRONMENT_SCOPED_KVM_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
            entry_name=entry_name,
        )
        hdrs = auth.set_header(
            self._auth, headers={'Accept': 'application/json', 'Content-Type': 'application/json'}
        )
        body = {'name': entry_name, 'value': updated_value}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def list_keys_in_an_environment_scoped_keyvaluemap(self, environment, startkey, count):
        uri = LIST_KEYS_IN_AN_ENVIRONMENT_SCOPED_KEYVALUEMAP_PATH.format(
            api_url=APIGEE_ADMIN_API_URL,
            org=self._org_name,
            environment=environment,
            name=self._map_name,
            startkey=startkey,
            count=count,
        )
        hdrs = auth.set_header(self._auth, headers={'Accept': 'application/json'})
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        # return KeyvaluemapsSerializer().serialize_details(resp, 'json', prefix=prefix)
        return resp

    def push_keyvaluemap(self, environment, file, secret=None):
        local_map = read_file(file, type='json')
        if secret:
            console.echo('Decrypting... ', end='', flush=True)
            local_map, decrypted_count = KeyvaluemapsSerializer().encrypt_decrypt_keyvaluemap(
                local_map, secret, encrypt=False
            )
            if decrypted_count:
                console.echo('Done')
            else:
                console.echo('None')
        self._map_name = local_map['name']
        try:
            remote_map = self.get_keyvaluemap_in_an_environment(environment).json()
            _, deleted_keys = KeyvaluemapsSerializer().diff_kvms(local_map, remote_map)
            local_map_updated = {
                'entry': [entry for entry in local_map['entry'] if entry not in remote_map['entry']]
            }
            if deleted_keys:
                self._Keyvaluemaps__delete_entries(environment, deleted_keys)
                console.echo('Removed entries.')
            if local_map_updated['entry']:
                for entry in tqdm(local_map_updated['entry'], **TQDM_KWARGS('Updating')):
                    self._Keyvaluemaps__create_or_update_entry(environment, entry)
                console.echo('Updated entries.')
            if not deleted_keys and not local_map_updated['entry']:
                console.echo('All entries up-to-date.')
        except HTTPError as e:
            if e.response.status_code != 404:
                raise e
            console.echo(f'Creating {self._map_name}')
            console.echo(
                self.create_keyvaluemap_in_an_environment(environment, json.dumps(local_map)).text
            )



import json
import os
from pathlib import Path


class InformalPullInterface:
    def __init__(self, auth, org_name, revision_number, environment, work_tree=None):
        self._auth = auth
        self._org_name = org_name
        if work_tree:
            if not os.path.exists(work_tree):
                os.makedirs(work_tree)
            self._work_tree = str(Path(work_tree).resolve())
        else:
            self._work_tree = os.getcwd()
        self._revision_number = revision_number
        self._environment = environment
        self._keyvaluemaps_dir = str(Path(self._work_tree) / 'keyvaluemaps' / environment)
        self._targetservers_dir = str(Path(self._work_tree) / 'targetservers' / environment)
        self._caches_dir = str(Path(self._work_tree) / 'caches' / environment)
        self._apiproxy_dir = str(Path(self._work_tree))
        self._zip_file = str(Path(self._apiproxy_dir).with_suffix('.zip'))

    def __call__(self, *args, **kwargs):
        self.pull(*args, **kwargs)

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def revision_number(self):
        return self._revision_number

    @revision_number.setter
    def revision_number(self, value):
        self._revision_number = value

    @property
    def environment(self):
        return self._environment

    @environment.setter
    def environment(self, value):
        self._environment = value

    @property
    def work_tree(self):
        return self._work_tree

    @property
    def keyvaluemaps_dir(self):
        return self._keyvaluemaps_dir

    @keyvaluemaps_dir.setter
    def keyvaluemaps_dir(self, value):
        self._keyvaluemaps_dir = str(Path(self._work_tree) / value / environment)

    @property
    def targetservers_dir(self):
        return self._targetservers_dir

    @targetservers_dir.setter
    def targetservers_dir(self, value):
        self._targetservers_dir = str(Path(self._work_tree) / value / environment)

    @property
    def caches_dir(self):
        return self._caches_dir

    @caches_dir.setter
    def caches_dir(self, value):
        self._caches_dir = str(Path(self._work_tree) / value / environment)

    @property
    def apiproxy_dir(self):
        return self._apiproxy_dir

    @apiproxy_dir.setter
    def apiproxy_dir(self, value):
        self._apiproxy_dir = str(Path(self._work_tree) / value)

    @property
    def zip_file(self):
        return self._zip_file

    @zip_file.setter
    def zip_file(self, value):
        self._zip_file = str(Path(self._apiproxy_dir) / value)



import click

from apigee import console
from apigee.auth import common_auth_options, gen_auth
from apigee.keystores.keystores import Keystores
from apigee.prefix import common_prefix_options
from apigee.silent import common_silent_options
from apigee.verbose import common_verbose_options


@click.group(help='A list of URIs used to create, modify, and delete keystores and truststores.')
def keystores():
    pass


def _create_a_keystore_or_truststore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def create(*args, **kwargs):
    _create_a_keystore_or_truststore(*args, **kwargs)


def _delete_a_keystore_or_truststore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def delete(*args, **kwargs):
    _delete_a_keystore_or_truststore(*args, **kwargs)


def _list_all_keystores_and_truststores(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    environment,
    prefix=None,
    **kwargs
):
    return Keystores(
        gen_auth(username, password, mfa_secret, token, zonename), org, None
    ).list_all_keystores_and_truststores(environment, prefix=prefix)


@keystores.command(help='Returns a list of all keystores and truststores in the environment.')
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-e', '--environment', help='environment', required=True)
def list(*args, **kwargs):
    console.echo(_list_all_keystores_and_truststores(*args, **kwargs))


def _get_a_keystore_or_truststore(
    username, password, mfa_secret, token, zonename, org, profile, name, environment, **kwargs
):
    return (
        Keystores(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_a_keystore_or_truststore(environment)
        .text
    )


@keystores.command(help='Returns a specific keystore or truststore in the environment.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get(*args, **kwargs):
    console.echo(_get_a_keystore_or_truststore(*args, **kwargs))


def _test_a_keystore_or_truststore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def test(*args, **kwargs):
    _test_a_keystore_or_truststore(*args, **kwargs)


def _get_cert_details_from_a_keystore_or_truststore(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    cert_name,
    **kwargs
):
    return (
        Keystores(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_cert_details_from_a_keystore_or_truststore(environment, cert_name)
        .text
    )


@keystores.command(help='Returns a specific cert from a keystore or truststore.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('--cert-name', help='cert name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get_cert(*args, **kwargs):
    console.echo(_get_cert_details_from_a_keystore_or_truststore(*args, **kwargs))


def _get_all_certs_from_a_keystore_or_truststore(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    prefix=None,
    **kwargs
):
    return Keystores(
        gen_auth(username, password, mfa_secret, token, zonename), org, name
    ).get_all_certs_from_a_keystore_or_truststore(environment, prefix=prefix)


@keystores.command(help='Returns all certs from a keystore or truststore.')
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get_all_certs(*args, **kwargs):
    console.echo(_get_all_certs_from_a_keystore_or_truststore(*args, **kwargs))


def _delete_cert_from_a_keystore_or_truststore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def delete_cert(*args, **kwargs):
    _delete_cert_from_a_keystore_or_truststore(*args, **kwargs)


def _export_a_cert(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    cert_name,
    **kwargs
):
    return (
        Keystores(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .export_a_cert(environment, cert_name)
        .text
    )


@keystores.command(help='Export a cert from a keystore or truststore.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('--cert-name', help='cert name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def export(*args, **kwargs):
    console.echo(_export_a_cert(*args, **kwargs))


def _upload_a_certificate_to_a_truststore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def upload_cert_to_truststore(*args, **kwargs):
    _upload_a_certificate_to_a_truststore(*args, **kwargs)


def _upload_a_jar_file_to_a_keystore(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def upload_jar_to_keystore(*args, **kwargs):
    _upload_a_jar_file_to_a_keystore(*args, **kwargs)


def _create_an_alias_by_generating_a_self_signed_certificate(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def create_alias(*args, **kwargs):
    _create_an_alias_by_generating_a_self_signed_certificate(*args, **kwargs)


def _list_aliases(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    prefix=None,
    **kwargs
):
    return Keystores(
        gen_auth(username, password, mfa_secret, token, zonename), org, name
    ).list_aliases(environment, prefix=prefix)


@keystores.command(help='Returns a list of all the aliases in the keystore.')
@common_auth_options
@common_prefix_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def list_aliases(*args, **kwargs):
    console.echo(_list_aliases(*args, **kwargs))


def _get_alias(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    alias_name,
    **kwargs
):
    return (
        Keystores(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .get_alias(environment, alias_name)
        .text
    )


@keystores.command(help='Returns details about an alias.')
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('--alias-name', help='alias name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def get_alias(*args, **kwargs):
    console.echo(_get_alias(*args, **kwargs))


def _update_the_certificate_in_an_alias(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def update_cert_in_alias(*args, **kwargs):
    _update_the_certificate_in_an_alias(*args, **kwargs)


def _generate_a_csr_for_an_alias(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def generate_csr_for_alias(*args, **kwargs):
    _generate_a_csr_for_an_alias(*args, **kwargs)


def _export_a_certificate_for_an_alias(
    username,
    password,
    mfa_secret,
    token,
    zonename,
    org,
    profile,
    name,
    environment,
    alias_name,
    **kwargs
):
    return (
        Keystores(gen_auth(username, password, mfa_secret, token, zonename), org, name)
        .export_a_certificate_for_an_alias(environment, alias_name)
        .text
    )


@keystores.command(
    help='Exports a certificate or certificate chain for the specified alias in a keystore.'
)
@common_auth_options
@common_silent_options
@common_verbose_options
@click.option('-n', '--name', help='keystore name', required=True)
@click.option('--alias-name', help='alias name', required=True)
@click.option('-e', '--environment', help='environment', required=True)
def export_cert_for_alias(*args, **kwargs):
    console.echo(_export_a_certificate_for_an_alias(*args, **kwargs))


def _delete_alias(*args, **kwargs):
    pass


@keystores.command(help='')
@common_auth_options
@common_silent_options
@common_verbose_options
def delete_alias(*args, **kwargs):
    _delete_alias(*args, **kwargs)



import json


class SharedflowsSerializer:
    def serialize_details(self, sharedflows, format, prefix=None):
        resp = sharedflows
        if format == 'text':
            return sharedflows.text
        sharedflows = sharedflows.json()
        if prefix:
            sharedflows = [
                sharedflow for sharedflow in sharedflows if sharedflow.startswith(prefix)
            ]
        if format == 'json':
            return json.dumps(sharedflows)
        elif format == 'table':
            pass
        return resp
